/*
 * 150452 scary halloween.cpp
 *
 * Created: 10-8-2015 15:18:55
 *  Author: Niek Laskarzewki
 */ 

#define F_CPU 10000000UL

#include <avr/io.h>
#include <avr/eeprom.h>
#include <util/delay.h>
#include <stdlib.h>

//I/O values for later use without confusion
int LEDPin = 0b1;
int clockPin = 0b10;
int dataPin = 0b100;
int busyPin = 0b1000;
int sensorPin = 0b10000;
int resetPin = 0b100000;

int count = 0;				//count is used to get into the scanVoice function
uint16_t voiceAmount = 0;		//the amount of voices which can be played
int sensorInit = 0;			//initialization count for the PIR sensor
int amount = 0;

int sdReset(void)			//resets the micro sd card module
{
	PORTB |= clockPin | resetPin;
	PORTB &= ~resetPin;
	_delay_ms(5);
	PORTB |= resetPin;
	_delay_ms(300);
	
	return 0;
}

int playVoice(unsigned int voice)	//sends data to sd card module to play a voice
{
	PORTB &= ~clockPin;
	_delay_ms(2);
	for(int pulse = 0; pulse < 16; pulse++)
	{
		PORTB &= ~clockPin;
		if (voice >> (15 - pulse) & 1 == 1)		//checks if data bit should be digital 1 or 0
		{
			PORTB |= dataPin;
		}
		else
		{
			PORTB &= ~dataPin;
		}
		
		_delay_us(200);
		PORTB |= clockPin;		//makes sure the clock pin is high
		_delay_us(200);
	}
	PORTB |= dataPin;		//makes sure the data pin is high
	
	return 0;
}

int scanVoice(void)
{
	voiceAmount = 0;
	while(1)
	{
		playVoice(amount);
		_delay_ms(2000);	
		if(!(PINB & busyPin))	//if the busy pin is low after 2 seconds, there is not a voice,
		{						//it stores the amount of voices in eeprom, and it can stop scanning
			amount = 0;
			eeprom_update_word(0,voiceAmount);
			break;
		}
		if(PINB & busyPin)	//if the busy pin is high after 2 seconds, there is a voice and it can continue scanning.
		{
			playVoice(0xFFFF);	//stop playing voice
			voiceAmount++;		//the amount of voices +1
		}
		_delay_ms(2000);
		amount++;
	}
	return 0;
}

int main(void)
{
	DDRB = 0b00100111;	//sets outputs
	sdReset();
	voiceAmount = eeprom_read_word((uint16_t*) 0);	//checks for the amount of voices in case it already scanned once
	if (voiceAmount == 0 || voiceAmount > 512)		//if the amount of voices = 0 or greater than max (512),
	{												//set the amount of voices to default (1) and update EEPROM.
		voiceAmount = 21;
		eeprom_update_word(0, voiceAmount);
	}
	PORTB |= LEDPin;	//blink LEDs when program is out of initialization.
	_delay_ms(500);
	PORTB &= ~LEDPin;
    while(1)
    {
        if(PINB & sensorPin)
		{
			sensorInit++;
			PORTB |= LEDPin;		//blinks when PIR sensor gives pulse
			while(PINB & sensorPin)
			{
				if (count == 15000)		//if the sensor pin is high after ~15 seconds or
				{											//the amount of voices = 0 (should not happen, just in case it does),
					PORTB &= ~LEDPin;						//the LEDs will blink to indicate the start of the scanning,
					_delay_ms(500);							//and the scanVoice function will be called
					PORTB |= LEDPin;
					_delay_ms(500);
					PORTB &= ~LEDPin;
					_delay_ms(500);
					PORTB |= LEDPin;
					_delay_ms(500);
					PORTB &= ~LEDPin;
					_delay_ms(500);
					PORTB |= LEDPin;
					scanVoice();
					voiceAmount = eeprom_read_word((uint16_t*) 0);	//reads the new value of the voices
					count = 0;
					sensorInit = 4;
					while(PINB & sensorPin)		//when scanning is done, the LEDs keep blinking to indicate it's done
					{
						PORTB |= LEDPin;
						_delay_ms(250);
						PORTB &= ~LEDPin;
						_delay_ms(250);
					}
				}
				count++;
				_delay_ms(1);
			}
			PORTB &= ~LEDPin;
		}
		
		if (sensorInit > 3)		//detects when the PIR sensor is out of initialization phase
		{
			while (1)	//cannot go back to initialization phase unless power is replugged
			{
				if (PINB & sensorPin)	//if the PIR sensor triggers a voice will be played
				{
					playVoice(rand() % voiceAmount);	//plays a random voice within the voice amount
					_delay_ms(200);		//makes sure the busy pin is high
					while(PINB & busyPin)	//waits until the busy pin is low
					{
						PORTB |= LEDPin;
						_delay_ms(100);
						PORTB &= ~LEDPin;
						_delay_ms(100);
					}
					_delay_ms(2000);
				}
			}
		}
    }
}